/*
 * This contains _two_ implementations of ps for Linux.  One uses the
 * traditional /proc virtual filesystem, and the other use the devps kernel
 * driver (written by Erik Andersen to avoid using /proc thereby saving 100k+).
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <stdarg.h>
#include <signal.h>
#include <time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#ifdef STANDALONE
#define kprintf printf
#else 
#include <fcntl.h>
#define LED_KPRINTF 8
static int kprintf (const char *fmt, ...)
{
    static char tmp_text[51];
	va_list ap;
    int fp;

    /* Open /dev/led file for LED Operation */
    fp = open("/dev/led", O_RDWR);
    if(!fp) {
        return -1;
    }
    
    memset (tmp_text, 0, sizeof (tmp_text));
    va_start(ap, fmt);
    vsnprintf(tmp_text, sizeof (tmp_text)-1, fmt, ap);
    va_end(ap);	
    ioctl(fp, LED_KPRINTF, &tmp_text);
    close(fp);

    return 0;
}
#endif //STANDALONE


/*
 * if ipaddr is in /proc/net/arp, return 1 and copy its mac to buf mac
    or return 0 and set mac buf to space 
*/
static int read_mac(const char* ipaddr, char* mac)
{
    int res = 0;
    char buf[4096];
    char *pos, *ptr;
    int fd;
    
    sprintf(mac, "%-17s", "                 ");
    fd = open("/proc/net/arp", O_RDONLY);
    if (fd<0)    return 0;
    res = read(fd, buf, sizeof(buf)-1);
    close(fd);
    if (res <=0) return 0;
    buf[res] =0;

    pos = strstr(buf, ipaddr);
    while ( pos )
    {
        ptr = strtok(pos, " \t\r\n");
        if( strcmp(ptr, ipaddr) ) {
            pos = strstr(ptr+1, ipaddr);
            continue;
        }
        ptr = strtok(NULL, " \t");
        ptr = strtok(NULL, " \t");
        ptr = strtok(NULL, " \t");
        strncpy(mac, ptr, 17);
        return 1;
    }

    return 0;	
}	

#define INVALID_SOCKET	(-1)
char lan_subnet[20];
char lan_ip[20];
	#include <netinet/in.h>
	#include <net/if.h>
	#include <unistd.h>
	#include <sys/socket.h>
	#include <sys/ioctl.h>

/*
This funciont return IP addres of a interface by input interface name, like
	return ip of br0, eth0, etc.
input:
	if_name : interface name
    type : 0 - ip, 1 - netmask, 2- broadcast ip
output:
	ip of the interface into add_return_ip,
	return 0 - means success, -1 error due ot error of giving NULL return address or other ioctl opreration error.
*/
int get_interface_ip_by_given_name(char  *if_name,char **addr_return_ip, int type)
{
	int fd;
	struct ifreq ifr;
	struct sockaddr_in *sin;
    long type_addr = SIOCGIFADDR;

    switch (type) {
    case 0:
        type_addr = SIOCGIFADDR;
        break;
    case 1:
        type_addr = SIOCGIFNETMASK;
        break;
    case 2:
        type_addr = SIOCGIFBRDADDR;
        break;
    }
	if (*addr_return_ip == NULL)
		return -1;
	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) >= 0) {
		ifr.ifr_addr.sa_family = AF_INET;
		strcpy(ifr.ifr_name, if_name);
		if (ioctl(fd, type_addr, &ifr) == 0) { //
			sin = (struct sockaddr_in *) &ifr.ifr_addr;
			sprintf(*addr_return_ip, "%s", inet_ntoa(sin->sin_addr));
//			syslog(LOG_ERR,"get_interface_ip_by_given_name(): %s",*addr_return_ip) ;

		} else {
//			kprintf("--ioctl failed, try again--\n");
			sleep(1);
			if (ioctl(fd, SIOCGIFADDR, &ifr) == 0) {
				sin = (struct sockaddr_in *) &ifr.ifr_addr;
				sprintf(*addr_return_ip, "%s", inet_ntoa(sin->sin_addr));
//				syslog(LOG_ERR,"get_interface_ip_by_given_name(): %s",*addr_return_ip) ;

			} else {
//				syslog(LOG_ERR,"get_interface_ip_by_given_name(): ioctl error") ;
				return -1;
			}
		}    
		close(fd);  
	} else {
//		syslog(LOG_ERR,"get_interface_ip_by_given_name(): Cannot get socket error") ;
		return -1;
	}
	return 0;
}

int get_lan_subnet (void)
{
	FILE *fp;
	char line[255];
	char *p;

	memset (lan_subnet, 0, sizeof (lan_subnet));
	memset (lan_ip, 0, sizeof (lan_ip));

    p= &lan_ip[0];
    get_interface_ip_by_given_name ("br0", &p, 0); //get br0 ip

    p= &lan_subnet[0];
    get_interface_ip_by_given_name ("br0", &p, 2); //get broadcast address

    if (!strlen (lan_ip)) {
        kprintf ("no lan ip got, use default\n");
        strcpy (lan_ip, "192.168.1.1");
    }
    if (!strlen (lan_subnet)) {
        kprintf ("no lan subnet got, use default\n");
        strcpy (lan_subnet, "192.168.1.255");
    }
    kprintf ("lan_ip %s,%s\n", lan_ip, lan_subnet);
	return 0;
}

int sock_138, sock_137;
struct sockaddr_in     addr_server_138, addr_server_137;

#define max(a,b) ((a)>(b))?(a):(b)
void *arp_creator (void)
{
	int             result;
	fd_set          fdset;								//file descripter set
	struct timeval  tval;								//timeout structure
	int             size;								//packet size
	char            response[20];
    int maxfdp1;
	int opt, num_try;
    struct sockaddr_in cli_addr;
    char ip[20];
    char mac[20];
    char cmd[100];
#ifndef STANDALONE
    sleep (30); //wait for /var/tmp/hosts be ready
    get_lan_subnet ();
#endif 
    num_try = 0;
    sock_138 = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (sock_138 == INVALID_SOCKET) {
        kprintf ("can't create socket for utg\n");
        return -1;
    }

    sock_137 = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (sock_137 == INVALID_SOCKET) {
        kprintf ("can't create socket for 137\n");
        close (sock_138);
        return -1;
    }
    //socket for smb server
    opt = 1;
    memset (&addr_server_138, 0, sizeof (addr_server_138));
    addr_server_138.sin_family = AF_INET;
    addr_server_138.sin_addr.s_addr = inet_addr (lan_subnet);
    addr_server_138.sin_port = htons(138);

    if (bind (sock_138, (struct sockaddr *) &addr_server_138, sizeof (addr_server_138)) < 0 ) 
    {
        kprintf ("bind 138 error\n");
        close (sock_138);
        close (sock_137);
        return (-1);
    }
    //socket for smb server
    opt = 1;
    memset (&addr_server_137, 0, sizeof (addr_server_137));
    addr_server_137.sin_family = AF_INET;
    addr_server_137.sin_addr.s_addr = inet_addr (lan_subnet);
    addr_server_137.sin_port = htons(137);
    if (bind (sock_137, (struct sockaddr *) &addr_server_137, sizeof (addr_server_137)) < 0 ) 
    {
        kprintf ("bind 137 error\n");
        close (sock_138);
        close (sock_137);
        return (-1);
    }

    while (1) { 
        FD_ZERO(&fdset);						//reset the file descripter set
        maxfdp1 = 0;
    	FD_SET(sock_138,    &fdset);						//add the socket to the set
    	FD_SET(sock_137,    &fdset);						//add the socket to the set
    
    	tval.tv_sec  = 120;
    	tval.tv_usec = 0;
        maxfdp1 = max (sock_138, sock_137) + 1;
    	result = select (maxfdp1, &fdset, NULL, NULL, &tval);
        if (result < 0)
    	{
            kprintf ("----select error in arp creator\n");
    		break;
    	}
        if (result == 0) { //timeout
            //kprintf ("I'm still working %d\n", num_try ++);
            sleep (10); //in case something wrong , don't running without a rest
            continue;
        }
        
        if (FD_ISSET (sock_138, &fdset)) {
            int len = sizeof (cli_addr);

            //read the response packet
        	memset (response, 0, sizeof(response));
        	size = recvfrom(sock_138, (char *)response, sizeof (response)-1, 0, (struct sockaddr *)&cli_addr, &len);
            if (size > 0)
            {
                memset (ip, 0, sizeof (ip));
                strcpy (ip, inet_ntoa (cli_addr.sin_addr));
                //kprintf ("138--from %s, %u\n", inet_ntoa (cli_addr.sin_addr), time (NULL));
                if (!read_mac (ip, mac)) {
                    memset (cmd, 0, sizeof (cmd));
                    snprintf (cmd, sizeof (cmd) -1, "arping -q -c 1 -I br0 %s&", ip);
                    system (cmd);
                    kprintf ("send arp to %s\n", ip);
                }
            }
        }
        if (FD_ISSET (sock_137, &fdset)) {
            int len = sizeof (cli_addr);

            //read the response packet
        	memset (response, 0, sizeof(response));
        	size = recvfrom(sock_137, (char *)response, sizeof (response)-1, 0, (struct sockaddr *)&cli_addr, &len);
            if (size > 0)
            {
                memset (ip, 0, sizeof (ip));
                strcpy (ip, inet_ntoa (cli_addr.sin_addr));
                //kprintf ("137--from %s, %u\n", inet_ntoa (cli_addr.sin_addr), time (NULL));
                if (!read_mac (ip, mac)) {
                    memset (cmd, 0, sizeof (cmd));
                    snprintf (cmd, sizeof (cmd) -1, "arping -q -c 1 -I br0 %s&", ip);
                    system (cmd);
                    kprintf ("send arp -> %s\n", ip);
                }
            }
        }
        sleep (5); //if too many packets, take a breathe
    }
    close (sock_138);
    close (sock_137);
	return 0;
}

#ifdef STANDALONE
int main(int argc, char **argv)
{
    kprintf ("----arp creator\n");
    strcpy (lan_subnet, "192.168.1.255");
	arp_creator ();
}
#endif //STANDALONE
