#ifndef LIB_COMM_H
#define LIB_COMM_H

/* The sys/un.h file does not define this. It is defined in the linux/un.h,
 * we don't want to use that one
 */
#define UNIX_PATH_MAX	108

#include <sys/un.h>
#include <sys/socket.h>
#include <sys/fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/types.h>
#include <unistd.h>

#define PC_IM_NAME          "im"
#define PC_IPQ_NAME         "ipq"
#define MAX_MESSAGE_LEN	    4096
#define MAX_SMSG_LEN        64
#define CM_MAX_MESSAGE_LEN	4096
#define RET_MSG_ERROR       -2
#define RET_MSG_BLOCK       -1
#define RET_MSG_OK          0

#define CMS_IP_NAME         "cms_ip" //ACTION_TEC_CMS
#ifdef ACTION_TEC_VERIZON_WRA
#define STANDBY_NAME         "standby_act" 
#endif
#define SOCKET_PATH "/var/etc/pc"

#define COMM_FAIL_SOCK			-1
#define COMM_FAIL_BIND			-2
#define COMM_FAIL_SECOND_INSTANCE	-3
#define COMM_FAIL_UNLINK		-4
#define COMM_TIMEOUT            3


int comm_start(char *name, int send_fl);
int comm_unlink(char *name);
int comm_sendto(int fd, const void *msg, size_t len, char *name);
#define comm_recv(fd, msg, len, flags) recv(fd, msg, len, flags)
#define comm_recvfrom(fd, msg, len, flags, from, fromlen) recvfrom(fd, msg, len, flags, from, fromlen)





//message defines
#define MSG_TRANS_UNKNOWN       0
#define MSG_TRANS_QUERY         1
#define MSG_TRANS_RESPONSE      2
#define MSG_TRANS_RESPONSE_CONT 3
#define MSG_TRANS_SET           4
#define MSG_TRANS_GROUP_BEGIN   5
#define MSG_TRANS_GROUP_END     6
#define MSG_TRANS_GROUP_ERROR   9
#define MSG_TRANS_APPLY         7
#define MSG_TRANS_REJECT        8

#define MAX_TRANS_VALUE_COLS	9


#define SECONDS_EXPIRE_VERDICT 2
#define MAX_MSGID	10
struct msg_base {
	char to[MAX_MSGID];
	char from[MAX_MSGID];
	unsigned long packet_id;	/* ID of queued packet */
	int seq;
	int msg_type;
	int expire; // used internaly to expire undeliverable messages

    int datalen;
	unsigned char *data;
	/* module is not supposed to touch anything below */
	struct msg_base *next;
	struct msg_base *prev;

};

struct msg_im_action {
    int seq;
    int action;
};

enum MSG_Type_Netacct {
    MSG_WEBCM,
    MSG_CLI,
    MSG_IPQ_ACT
};

int     msg_sendOut(struct msg_base *msg, char *name);
void    msg_free (struct msg_base *msg);
void	flush_messages(void);
int	    msg_iterate_queue(void);
int	    msg_get_seq();
struct  msg_base* find_msg (int seq);
struct  msg_base* new_msg (int msg_type, char *data, int datalen, char *to, char *from);
int     msg_send(struct msg_base *msg);
void    msg_flush(void);


#endif //LIB_COMM_H
