/*
 * Author: Paul.Russell@rustcorp.com.au and mneuling@radlogic.com.au
 *
 * Based on the ipchains code by Paul Russell and Michael Neuling
 *
 * (C) 2000-2002 by the netfilter coreteam <coreteam@netfilter.org>:
 * 		    Paul 'Rusty' Russell <rusty@rustcorp.com.au>
 * 		    Marc Boucher <marc+nf@mbsi.ca>
 * 		    James Morris <jmorris@intercode.com.au>
 * 		    Harald Welte <laforge@gnumonks.org>
 * 		    Jozsef Kadlecsik <kadlec@blackhole.kfki.hu>
 *
 *	iptables -- IP firewall administration for kernels with
 *	firewall table (aimed for the 2.3 kernels)
 *
 *	See the accompanying manual page iptables(8) for information
 *	about proper usage of this program.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <iptables.h>
#ifdef ACTION_TEC
void clean_up_old_icmp(void);	
int clean_up_old_icmp_flag=0;
#endif

int
main(int argc, char *argv[])
{
	int ret;
	char *table = "filter";
	iptc_handle_t handle = NULL;

	program_name = "iptables";
	program_version = IPTABLES_VERSION;

#ifdef NO_SHARED_LIBS
	init_extensions();
#endif

	ret = do_command(argc, argv, &table, &handle);
	if (ret)
		ret = iptc_commit(&handle);

	if (!ret)
		fprintf(stderr, "iptables: %s\n",
			iptc_strerror(errno));

#ifdef ACTION_TEC
	if(clean_up_old_icmp_flag)
		clean_up_old_icmp();
#endif

	exit(!ret);
}

#ifdef ACTION_TEC
void clean_up_old_icmp()
{
	static struct timeval timenow;		/* Current time */
	FILE *fproc;
//    gettimeofday(&timenow, NULL);
	if (fproc=fopen("/proc/sys/net/ipv4/netfilter/nat_masq_starting_time","w+")){
		#define MAX_BYTES_OF_LONG 4
		char str[MAX_BYTES_OF_LONG+1],*pstr;
		long templ;
		pstr=(char *)&str[0];
//		ltoa(timenow.tv_sec,(char *)&str[0],10);
		fgets(pstr,MAX_BYTES_OF_LONG,fproc);
		templ=atol(pstr)+1;
		sprintf(pstr,"%i",templ);
        fwrite(pstr,1,strlen(pstr),fproc);
		fclose(fproc);
//		printf("\nNAT COUNTER == %i\n",atol(pstr));
	}

}
	
#endif

