/*
  msgsniff.c

  Sniff chat messages (AIM, ICQ, IRC, MSN, Yahoo) on a network.

  $Id: msgsniff.c,v 1.1.1.1 2005/03/22 18:30:58 mfpeng Exp $
*/

#include <sys/types.h>
#include <sys/queue.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <regex.h>
#include <err.h>
#include "buf.h"
#include <netinet/in.h> 
#include <syslog.h>
#include <linux/if.h>
#include <linux/netfilter_ipv4/ipt_ULOG.h>
#include <linux/udp.h>
#include <linux/tcp.h>
#include <linux/ip.h>
#define pletohs(p)	((u_short)                         \
			 ((u_short)*((u_char *)p+1)<<8|    \
			  (u_short)*((u_char *)p+0)<<0))
     
#define pletohl(p)	((u_int32_t)*((u_char *)p+3)<<24|  \
			 (u_int32_t)*((u_char *)p+2)<<16|  \
			 (u_int32_t)*((u_char *)p+1)<<8|   \
			 (u_int32_t)*((u_char *)p+0)<<0)

#define pntohs(p)	((u_short)			   \
			 ((u_short)*((u_char *)p+1)<<0|    \
			  (u_short)*((u_char *)p+0)<<8))
			 
#define pntohl(p)	((u_int32_t)*((u_char *)p+3)<<0|   \
			 (u_int32_t)*((u_char *)p+2)<<18|  \
			 (u_int32_t)*((u_char *)p+1)<<16|  \
			 (u_int32_t)*((u_char *)p+0)<<24)

#define PACKET_OUT  0
#define PACKET_IN   1
/* Responses from hook functions. */
#define NF_DROP 0
#define NF_ACCEPT 1
#define NF_STOLEN 2
#define NF_QUEUE 3
#define NF_REPEAT 4
#define NF_MAX_VERDICT NF_REPEAT
#define MAX_CLIENTS 20
struct client_info {
	char	    *nick;
	char	    *peer;
	int 	    type;
    char        ver[10];
    int         logoned;
    char        ip_str[16];
	in_addr_t   ip;
    int         port;
    int         useExistingNick; //for aim only, after signon it will use another port to send msg
    time_t      active_time;
	struct client_info* next;
	struct client_info* prev;
};

struct client_info *client_list = NULL;
void init_client_list (void)
{
    client_list = NULL;
}
void insert_client (struct client_info *newclient)
{
    if (!newclient) return;
    
    newclient->next = client_list;
    if (client_list) {
        client_list->prev = newclient;
    }
    client_list = newclient;
    client_list->prev = NULL;
}

void remove_client (struct client_info *client)
{
    if (!client) return;
    if (client == client_list) client_list = client->next;
    
    if (client->prev)
        client->prev->next = client->next;
    if (client->next) {
        client->next->prev = client->prev;
    }

    if (client->nick) free (client->nick);
    if (client->peer) free (client->peer);
    free (client);
}

int		Opt_invert = 0;
regex_t	       *pregex = NULL;
#define MAX_WORD 255
#define MAX_CONTENT 500
#define MAX_NICK 100

enum IM_TYPE {
    MSN_IM,
    AOL_IM,
    YAHOO_IM,
    ICQ_IM,
    MSN_CHAT,
    AOL_CHAT,
    YAHOO_CHAT,
    ICQ_CHAT
};

enum EVENT_TYPE {
    EVT_LOGON,
    EVT_LOGOFF,
    EVT_MSG_INVITE,
    EVT_MSG_INVITED,
    EVT_MSG_OUT,
    EVT_MSG_IN,
    EVT_CHAT_JOIN,
    EVT_CHAT_EXIT,
    EVT_CHAT_IN,
    EVT_CHAT_OUT,
    EVT_NOTEXIST=255
};

struct Event_String {
    int event_type;
    char *string;
};

struct Event_String event_strings[] = {
    {EVT_LOGON,         "LOGON"},
    {EVT_LOGOFF,        "LOGOFF"},
    {EVT_MSG_INVITE,    "MSG_INVITE"},
    {EVT_MSG_INVITED,   "MSG_INVITED"},
    {EVT_MSG_OUT,       "MSG_OUT"},
    {EVT_MSG_IN,        "MSG_IN"},
    {EVT_CHAT_JOIN,     "CHAT_JOIN"},
    {EVT_CHAT_EXIT,     "CHAT_EXIT"},
    {EVT_CHAT_IN,       "CHAT_IN"}, 
    {EVT_CHAT_OUT,      "CHAT_OUT"},
    {EVT_NOTEXIST,      NULL}
};

/*
 * if ip_src has logonned user, return 1 with the userid in logonuserid
   or return 0;
*/
#define MAX_LEN_USERID      30
#define FILE_LOGONTABLE     "/var/etc/pc/logontable"
int get_logonuserid (char *ip_src, char *logonuserid)
{
    FILE *fp;
    char *ip;
    char *logonoff;
    char buf[100];
    char *userid = NULL;

    fp = fopen (FILE_LOGONTABLE,  "r");
    if (fp) {
        while (fgets(buf, sizeof(buf)-1, fp) != NULL) {
            //check if the src ip has logon or not
            ip =  strtok (buf, " \t\n");
            logonoff = strtok (NULL, " \t\n");
            if (ip && 
                (strcmp (ip_src, ip) == 0) &&
                logonoff && 
                (logonoff[0] == '1')) {
                userid = strtok (NULL, " \t\n");
                break;
            }
            memset (buf, 0, sizeof (buf));
        }
        fclose (fp);
    }

    if (userid) {
        strncpy (logonuserid, userid, MAX_LEN_USERID);
        return 1;
    }

    return 0;
}
/*
 * 
*/
void get_userid (char **nick, char *ip_local)
{
    char word[MAX_LEN_USERID];

    if (!*nick || !strcmp (*nick, "null")) {
        if (*nick) free (*nick);
        memset (word, 0, sizeof (word));
        get_logonuserid (ip_local, word);
        if (strlen (word) == 0) {
            *nick = strdup ("unkonwn");
        }
        else {
            *nick = strdup (word);
        }
    }
}

/*
 * check whether ip_local can talk with (type,name) or not
*/
int checkBlockIMList (int type, char *name, char *ip_local) 
{
    char buf[100];
    int ret = 1; //NF_ACCEPT
    FILE *fp = fopen ("/var/etc/pc/im_block", "r");
    char *im_type;
    char *im_blocked;
    char *user_level;

    if (!fp) return ret; //no block defined
    
    while (fgets (buf, sizeof (buf) -1, fp) != NULL) {
        im_type = strtok (buf, " \t\n");
        im_blocked = strtok (NULL, " \t\n");
        user_level = strtok (NULL, " \t\n");
        if (im_type && im_blocked) {
            if ((atoi (im_type) == type) && 
                (strcmp (name, im_blocked) == 0)) {
                //add code to use ip_local to get the user_level using this pc
                syslog (LOG_INFO, "***********MATCH, BLOCKED %d,%s,%s", type, name, ip_local);
                ret = NF_DROP;
                break;
            }
        }
    }
    fclose (fp);
    return ret;
}
                        
#define PREFIX_LOG_IM      "syslog: IM_LOG:"
#define FILE_REMOTE_SERVER  "/var/etc/pc/remote_server"
#define MAX_NICKNAME   100
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <time.h>
/* udp socket for logging to remote host */
static int remotefd = -1;
/* what port to log to? */
static int RemotePort = 514;

void send_im_event (int im_type, int event_type, char *nickname, int verdict, char *peername, char *msg) 
{
    struct sockaddr_in remoteaddr;
    FILE *fp;
    char buf[30];
    char *ip;
    char msg_buf[2048];
    char *portstr;
    int  port = 0;
    char *evt_str = NULL;
    char *im_str = NULL;
    char *verdict_str = "ACCEPT";
    int i;

    if (remotefd < 0) {
        remotefd = socket(AF_INET, SOCK_DGRAM, 0);

        if (remotefd < 0) {
            syslog (LOG_ERR, "syslogd: cannot create socket");
            return;
        }
    }
    
    i= 0;
    while (event_strings[i].event_type != EVT_NOTEXIST) {
        if (event_strings[i].event_type == event_type) {
            evt_str = event_strings[i].string;
            break;
        }
        i++;
    }

    if (!evt_str) {
        syslog (LOG_ERR, "No such event %d", event_type);
        return;
    }
    
    switch (im_type) {
    case MSN_IM:
    case MSN_CHAT:
        im_str = "MSN";
        break;
    case AOL_IM: 
    case AOL_CHAT:
        im_str = "AOL";
        break;
    case YAHOO_IM:
    case YAHOO_CHAT:
        im_str = "YAHOO";
        break;
    case ICQ_IM: 
    case ICQ_CHAT:
        im_str = "ICQ";
        break;
    default:
        im_str = NULL;
    }
    if (!im_str) {
        syslog (LOG_ERR, "No such im type %d", im_type);
        return;
    }

    if (verdict != NF_ACCEPT) {
        verdict_str = "BLOCKED";
    }
    memset (msg_buf, 0, sizeof (msg_buf));
    switch (event_type) {
    case EVT_MSG_OUT:
        snprintf (msg_buf, sizeof (msg_buf) -1, "%s %s %s %s %s->%s %s", 
                  PREFIX_LOG_IM, evt_str, im_str, verdict_str, nickname, peername, msg);
        break;

    case EVT_CHAT_OUT:
        snprintf (msg_buf, sizeof (msg_buf) -1, "%s %s %s %s %s %s %s", 
                  PREFIX_LOG_IM, evt_str, im_str, verdict_str, nickname, peername, msg);
        break;

    case EVT_CHAT_JOIN:
    case EVT_CHAT_EXIT:
        snprintf (msg_buf, sizeof (msg_buf) -1, "%s %s %s %s %s %s", 
                  PREFIX_LOG_IM, evt_str, im_str, verdict_str, nickname, peername);
        break;

    case EVT_MSG_IN:
        snprintf (msg_buf, sizeof (msg_buf) -1, "%s %s %s %s %s<-%s %s", 
                  PREFIX_LOG_IM, evt_str, im_str, verdict_str, nickname, peername, msg);
        break;

    case EVT_CHAT_IN:
        snprintf (msg_buf, sizeof (msg_buf) -1, "%s %s %s %s %s %s %s", 
                  PREFIX_LOG_IM, evt_str, im_str, verdict_str, nickname, peername, msg);
        break;

    default:
        snprintf (msg_buf, sizeof (msg_buf) -1, "%s %s %s %s %s", 
                  PREFIX_LOG_IM, evt_str, im_str, verdict_str, nickname);
        break;

    }
    fp = fopen (FILE_REMOTE_SERVER, "r");
    if (fp) {
        memset (buf, 0, sizeof (buf));
        fgets (buf, sizeof (buf) -1, fp);
        ip = strtok (buf, " \n\t");
        if (ip) {
            portstr = strtok (NULL, " \n\t");
            if (portstr) port = atoi (portstr);
            if (!port) port =  RemotePort;

            memset(&remoteaddr, 0, sizeof (struct sockaddr_in));
            remoteaddr.sin_family = AF_INET;
            remoteaddr.sin_port = htons(port);
            inet_pton (AF_INET, ip, &remoteaddr.sin_addr);
            if (remoteaddr.sin_addr.s_addr != 0) {
                sendto (remotefd, msg_buf, strlen (msg_buf), 0, (struct sockaddr *)&remoteaddr, sizeof (struct sockaddr_in ));
            }
        }
        fclose (fp);
    }
}

char *
timestamp(void)
{
	static char stamp[32];
	struct tm *tm;
	time_t now;

	time(&now);
	tm = localtime(&now);
	strftime(stamp, sizeof(stamp), "%b %e %T", tm);

	return (stamp);
}

int
regex_match(char *string)
{
	return (pregex == NULL ||
		((regexec(pregex, string, 0, NULL, 0) == 0) ^ Opt_invert));
}

struct flap {
	u_char	start;
	u_char	channel;
	u_short	seqnum;
	u_short	datalen;
};

struct snac {
	u_short		family;
	u_short		subtype;
	u_short		flags;
	u_int32_t	reqid;
};

int
process_aim(struct client_info *info, u_char *data, int len, int direction, char *ip_local)
{
	struct buf *msg, buf;
	struct flap *flap;
	struct snac *snac;
	u_char c, *p;
	int i, reply;
    int ret = NF_ACCEPT; 
    char word[MAX_WORD];
    char content[MAX_CONTENT];
    static int aim_just_logoff = 0;

	buf_init(&buf, data, len);

	syslog (LOG_INFO, "%d,%d--------buflen=%d,%d------aimdata", direction, len,
            buf_len(&buf), sizeof (*flap));
	if (buf_cmp(&buf, "FLAPON\r\n\r\n", 10) == 0)
		buf_skip(&buf, 10);

	while (buf_len(&buf) >= sizeof(*flap)) {
		flap = (struct flap *)buf_ptr(&buf);
		flap->datalen = ntohs(flap->datalen);
		i = sizeof(*flap) + flap->datalen;

		if (flap->channel == 0x04 && flap->start == 0x2a) { //close connection
            if (direction == PACKET_OUT && info->logoned) { //ignore it if not receiving the second new connection 
                if (!aim_just_logoff) {
                    get_userid (&info->nick, ip_local);
                    send_im_event (info->type, EVT_LOGOFF, info->nick, NF_ACCEPT, "", "");            
                    aim_just_logoff = 1;
                }
                remove_client (info);
            }
            break;
        }
		else if (flap->channel == 0x01 && flap->start == 0x2a) { //new connection
            //aim has 2 new connection cmds.
            if (direction == PACKET_OUT && info->useExistingNick) {
                info->logoned = 1;
                aim_just_logoff = 0;
            }
            break;
        }
		if ((msg = buf_tok(&buf, NULL, i)) == NULL)
			break;

		buf_skip(msg, sizeof(*flap));
		snac = (struct snac *)buf_ptr(msg);
		
//        syslog (LOG_INFO, "--------CHANNEL=%d,Snac=%08x,%08x\n", 
//                flap->channel, ntohs (snac->family), ntohs (snac->subtype));
		if (flap->start != 0x2a)
			continue;
		
		if (flap->channel == 0x01) {
			if (buf_cmp(msg, "\x00\x00\x00\x01\x00\x01\x00", 7) == 0) { //don't what does this mean.
				buf_skip(msg, 7);
				buf_get(msg, &c, 1);
                memset (word, 0, sizeof (word));
                if (buf_copybuf (msg, 0, c, word, sizeof (word)) > 0) {
                    if (info->nick) free(info->nick);
                    info->nick = strdup(word);
                }
                buf_skip(msg, 3);
                buf_get(msg, &c, 1);
                buf_skip(msg, c + 4);
    
                if (buf_cmp(msg, "ICQ", 3) == 0)
                    info->type = ICQ_IM;
                else info->type = AOL_IM;
    
                get_userid (&info->nick, ip_local);
                ret = checkBlockIMList (info->type, info->nick, ip_local);
                if (ret == NF_ACCEPT && info->type == AOL_IM) {
                    info->logoned = 1;
                }
                syslog (LOG_INFO, "%s SIGNON %s", info->type==AOL_IM?"AIM":"ICQ", info->nick);
                send_im_event (info->type, EVT_LOGON, info->nick, ret, "", "");
            }
		}
		else if (flap->channel == 0x02) {
			if (buf_cmp(msg, "toc_send_im ", 12) == 0) {
				buf_skip(msg, 12);
				
                if (buf_copyword(msg, " ", 1, word, sizeof (word)) == 0)
					continue;
				
				buf_skip(msg, 1);
				
				if (buf_len(msg) < 3) continue;
				msg->end -= 2;

                buf_strcopy (msg, content, sizeof (content));
                syslog (LOG_INFO, "%s %s %s > %.*s: %s\n",
                       timestamp(), info->type== AOL_IM?"AIMM":"ICQ", info->nick,
                       strlen (word), word, content);
                send_im_event (info->type, EVT_MSG_OUT, info->nick, ret, word, content);
			}
			else if (buf_cmp(msg, "IM_IN:", 6) == 0) {
				buf_skip(msg, 6);
				
                if (buf_copyword(msg, ":", 1, word, sizeof (word)) == 0)
					continue;
				
				buf_skip(msg, 2);

                buf_strcopy (msg, content, sizeof (content));
                syslog (LOG_INFO, "%s AAIM %.*s > %s: %s\n",
                       timestamp(), strlen (word),
                       word, info->nick, content);
                send_im_event (info->type, EVT_MSG_IN, info->nick, ret, word, content);
			}
			else if (ntohs(snac->family) == 0x04) {
				
				if (ntohs(snac->subtype) == 0x06)
					reply = 0;
				else if (ntohs(snac->subtype) == 0x07)
					reply = 1;
				else continue;
				
				buf_skip(msg, sizeof(*snac) + 8);
				buf_get(msg, &c, 1);
				
                if (buf_copybuf (msg, 0, c, word, sizeof (word)) < 0) 
					continue;

				/* XXX - ugh, this is totally bogus. help! */
				if (buf_cmp(msg, "\x00\x02", 2) == 0) {
					buf_skip(msg, 17);
					while (buf_cmp(msg, "\x00", 1) == 0)
						buf_skip(msg, 1);
				}
				else if (buf_cmp(msg, "\x00\x05", 2) == 0) {
					buf_skip(msg, 97);
				}
				else if (buf_cmp(msg, "\x00\x00", 2) == 0) {
					if (buf_skip(msg, 145) < 0)
						buf_skip(msg, 57);
				}

                buf_strcopy (msg, content, sizeof (content));
                if (strlen (content) ) {
					if (reply) {
						syslog (LOG_INFO, "%s %s %.*s > %s: %s\n",
						       timestamp(), info->type == ICQ_IM?"ICQ":"AIMMM",
						       strlen(word),
						       word ,
						       info->nick, content);
					}
					else syslog (LOG_INFO, "%s %s %s > %.*s: %s\n",
						    timestamp(), info->type == ICQ_IM?"ICQ":"AIMMM",
						    info->nick, strlen(word),
						    word, content);
				}
                send_im_event (info->type, reply?EVT_MSG_IN:EVT_MSG_OUT, info->nick, ret, word, content);
			}
			else if (ntohs(snac->family) == 0x17 && direction == PACKET_OUT) { //AIM signon
				if (ntohs(snac->subtype) == 0x06) {  //sign-on
                    int scrlen;

    				buf_skip(msg, sizeof(*snac)); 
    				buf_get(msg, &scrlen, 2);
    				scrlen = ntohs (scrlen);
                    if (buf_copybuf (msg, 0, scrlen, word, sizeof (word)) < 0) 
    					continue;

                    if (info->nick) free(info->nick);
                    info->nick = strdup(word);
                    get_userid (&info->nick, ip_local);
                    ret = checkBlockIMList (info->type, info->nick, ip_local);
                    syslog (LOG_INFO, "%s SIGNONN %s", info->type==AOL_IM?"AIM":"ICQ", info->nick);
                    send_im_event (info->type, EVT_LOGON, info->nick, ret, "", "");
                }
			}
		}

	}
	return ret; //(len - buf_len(&buf));
}
			
int
process_irc(struct client_info *info, u_char *data, int len, int direction, char *ip_local)
{
	struct buf *line, buf;
	char *p;
	int i;
    int ret = NF_ACCEPT;
    char word[MAX_WORD], prefix[MAX_WORD];
    char msg[MAX_WORD];

    memset (word, 0, sizeof (word));
    memset (prefix, 0, sizeof (prefix));
	syslog (LOG_INFO, "%d,%d-----IRCdata", direction, len);
	buf_init(&buf, data, len);
	while ((i = buf_index(&buf, "\n", 1)) >= 0) {
		line = buf_tok(&buf, NULL, i);
		buf_skip(&buf, 1); //why?
		
		if (line->base[line->end-1] == '\r')
			line->end--;
		
		if (buf_cmp(line, ":", 1) == 0) {
			buf_skip(line, 1);

            if (buf_copyword(line, " ", 1, prefix, sizeof (prefix)) == 0)
				continue;
            if ((p = strchr (prefix, '!')) == NULL) 
				continue;
			p[0] = 0;
		}
		else  {
            memset (prefix, 0, sizeof (prefix));
        }

		if (buf_cmp(line, "JOIN ", 5) == 0) {
            if (direction == PACKET_OUT) {
    			buf_skip(line, 5);
    			if (buf_cmp(line, ":", 1) == 0)
    				buf_skip(line, 1);
                else 
                    buf_skip(line, 2); //skip %#

                if (buf_copyword(line, " ", 1, prefix, sizeof (prefix)) == 0)
                    continue;
    			
                get_userid (&info->nick, ip_local);
                ret = checkBlockIMList (MSN_CHAT, prefix, ip_local);
                syslog (LOG_INFO, "JOIN %s IRC *** %.*s ", timestamp(),
    			       strlen(prefix), prefix);
                
    			syslog (LOG_INFO, "%s has joined channel %s\n",
    			       info->nick, prefix);
                send_im_event (MSN_CHAT, EVT_CHAT_JOIN, info->nick, ret, prefix, "");
            }
		}
		else if (buf_cmp(line, "PART ", 5) == 0) {
            if (direction == PACKET_OUT) {
    			buf_skip(line, 5);
    			if (buf_cmp(line, ":", 1) == 0)
    				buf_skip(line, 1);
    
                if (buf_copyword(line, " :", 2, word, sizeof (word)) == 0)
                    continue;
    			
    			syslog (LOG_INFO, "%s IRC *** %s has left channel %s\n",
    			       timestamp(), info->nick, word);
            }
		}
		else if (buf_cmp(line, "QUIT ", 5) == 0) {
            if (direction == PACKET_OUT) {
    			buf_skip(line, 5);
    			if (buf_cmp(line, ":", 1) == 0)
    				buf_skip(line, 1);
    
    			syslog (LOG_INFO, "%s IRC *** Signoff: %s\n",
    			       timestamp(), info->nick, buf_ptr(line));
                send_im_event (MSN_CHAT, EVT_CHAT_EXIT, info->nick, NF_ACCEPT, buf_ptr(line), "");
            }
		}
		else if (buf_cmp(line, "NICK ", 5) == 0) {
            if (direction == PACKET_OUT) {
    			buf_skip(line, 5);
    			if (buf_cmp(line, ":", 1) == 0)
    				buf_skip(line, 1);
    			
    			if (strlen (prefix) > 0) {
    				syslog (LOG_INFO, "%s IRC *** %.*s is now known as %.*s\n",
    				       timestamp(),
    				       strlen (prefix), prefix,
    				       buf_len(line), buf_ptr(line));
    			}
    			else {
    				if (info->nick) free(info->nick);
    				info->nick = buf_strdup(line);
                    get_userid (&info->nick, ip_local);
                    syslog (LOG_INFO, "NICK name = %s", info->nick);
    			}
            }
		}
		else if (buf_cmp(line, "PRIVMSG ", 8) == 0) {
			buf_skip(line, 8);
            if (buf_copyword(line, " :", 2, word, sizeof (word)) == 0)
				continue;

            if (buf_cmp (line, "%#" ,2) == 0) {
                buf_skip(line, 2);
            }
			
            p = buf_ptr (line);
            if (strncmp(p + 1, "ACTION ", 7) == 0) { //don't understand yet
                syslog (LOG_INFO, "%s IRC * Action: ",
                       timestamp());
                
                if (strlen (prefix) > 0) {
                    syslog (LOG_INFO, "%s %s\n",
                           prefix,
                           p + 8);
                }
                else syslog (LOG_INFO, "%s %s\n",
                        info->nick, p + 8);
            }
            else {
                get_userid (&info->nick, ip_local);

                if (strlen (prefix) > 0) {
                    syslog (LOG_INFO, "%s IRC1 %s > ",
                           timestamp(),
                           prefix);
                }
                else syslog (LOG_INFO, "%s IRC2 %s > ",
                        timestamp(),
                        info->nick);
                ret = checkBlockIMList (MSN_CHAT, word, ip_local);
                syslog (LOG_INFO, "%s: %s\n", word ,buf_ptr (line));
                send_im_event (MSN_CHAT, EVT_CHAT_IN, info->nick, 
                               ret, word, buf_ptr (line));
            }
		}
	}
	return ret; 
}

int
process_msn(struct client_info *info, u_char *data, int len, int direction, char *ip_local)
{
	struct buf *line, buf;
	char *p;
	int i, reply;
    char word[MAX_WORD];
    int ret = NF_ACCEPT;

    memset (word, 0, sizeof (word));
    memcpy (word, data, 20);
	syslog (LOG_INFO, "%d,%d-----MSNdata=%s", direction, len, word);
	buf_init(&buf, data, len);
	while ((i = buf_index(&buf, "\r\n", 2)) >= 0) {
		line = buf_tok(&buf, NULL, i);
		buf_skip(&buf, 2);
		
		if (buf_cmp(line, "USR ", 4) == 0) {
			if ((i = buf_index(line, "MD5 ", 4)) > 0) {
				buf_skip(line, i + 4);
				
				if (buf_cmp(line, "I ", 2) == 0) {
					buf_skip(line, 2);
					if (info->nick != NULL)
						free(info->nick);
					info->nick = buf_strdup(line);
                    syslog (LOG_INFO, "got msn usr md5 %s", info->nick);
				}
			}
            else {
                if (direction == PACKET_IN) {
                    buf_skip(line, 4);
                    if (buf_copyword(line, " ", 1, word, sizeof (word)) == 0)
                        continue;
                    if (buf_copyword(line, " ", 1, word, sizeof (word)) == 0)
                        continue;
                    syslog (LOG_INFO, "usr %s", word);
                    if ((strcmp (word, "OK") == 0) && !info->logoned ) {
                        if (buf_copyword(line, " ", 1, word, sizeof (word)) == 0)
                            continue;
                        if (info->nick != NULL)
                            free(info->nick);
                        info->nick = strdup(word);
                        syslog (LOG_INFO, "got logon screen name %s", info->nick);
                        send_im_event (MSN_IM, EVT_LOGON, info->nick, NF_ACCEPT, "", "");
                        info->logoned = 1;
                    }
                }
            }
		}
		else if (buf_cmp(line, "CAL ", 4) == 0) {
            if (direction == PACKET_OUT) {
                buf_skip(line, 4);
                if (buf_copyword(line, " ", 1, word, sizeof (word)) == 0)
                    continue;
                if (buf_copyword(line, " ", 1, word, sizeof (word)) == 0) {
                    strncpy (word, buf_ptr (line), sizeof (word) -1);
                }
                syslog (LOG_INFO, "got CALL screen name %s", word);
                ret = checkBlockIMList (MSN_IM, word, ip_local);
                send_im_event (MSN_IM, EVT_MSG_INVITE, word, ret, "", "");
            }
		}
		else if (buf_cmp(line, "VER ", 4) == 0) {
			buf_skip(line, 4);
            if (buf_copyword(line, " ", 1, word, sizeof (word)) == 0)
                continue;
            if (buf_copyword(line, " ", 1, word, sizeof (word)) == 0)
                continue;
            memset (info->ver, 0, sizeof (info->ver));
            strncpy (info->ver, word, sizeof (info->ver) - 1);
            syslog (LOG_INFO, "got VER %s", info->ver);
		}
		else if (buf_cmp(line, "IRO ", 4) == 0) {
            syslog (LOG_INFO, "IRO LINE %s", buf_ptr (line));
			if ((i = buf_rindex(line, "1 ", 2)) < 0)
                continue;
			buf_skip(line, i + 2);
            if (buf_copyword(line, " ", 1, word, sizeof (word)) == 0)
                continue;
			if (info->peer != NULL) free(info->peer);
			info->peer = strdup (word);
            syslog (LOG_INFO, "got IRO from %s", info->peer);
            ret = checkBlockIMList (MSN_IM, info->peer, ip_local);
            send_im_event (MSN_IM, EVT_MSG_INVITED, info->peer, ret, "", "");
		}
		else if (buf_cmp(line, "JOI ", 4) == 0) {
			buf_skip(line, 4);
            if (buf_copyword(line, " ", 1, word, sizeof (word)) == 0)
                continue;
            syslog (LOG_INFO, "got JOI from %s", word);
            if (info->peer) free (info->peer);
            info->peer = strdup (word);
            ret= checkBlockIMList (MSN_IM, word, ip_local);
		}
		else if (buf_cmp(line, "MSG ", 4) == 0) {
			buf_skip(line, 4);
			reply = (direction == PACKET_IN);

            if (buf_copyword(line, " ", 1, word, sizeof (word)) == 0)
                continue;
            
            if (strcmp (word, "Hotmail") == 0) { //ignore this line
                continue;
            }

            if (reply) {
				if (info->peer != NULL) free(info->peer);
				info->peer = strdup(word);
                syslog (LOG_INFO, "got reply from %s", info->peer);
                ret= checkBlockIMList (MSN_IM, word, ip_local);
			}

            if ((i = buf_rindex(line, " ", 1)) < 0)
                continue;

            buf_skip(line, i + 1);
            p = buf_strdup(line);
            i = atoi(p); free(p);
            if (i <= 0) continue;

            if ((line = buf_tok(NULL, NULL, i)) == NULL)
                    continue;

            if (buf_index(line, "Content-Type: text/plain", 24) > 0) {
                if ((i = buf_rindex(line, "\r\n\r\n", 4)) < 0)
                        continue;

                buf_skip(line, i + 4);
                p = buf_strdup(line);

                if (reply) {
                    syslog (LOG_INFO, "%s MSN %s > %s: %s\n",
                           timestamp(), info->peer,
                           info->nick, p);
                    send_im_event (MSN_IM, EVT_MSG_IN, info->nick, NF_ACCEPT, info->peer, p);
                }
                else {
                    syslog (LOG_INFO, "%s MSN %s > %s: %s\n",
                            timestamp(), info->nick,
                            info->peer, p);
                    send_im_event (MSN_IM, EVT_MSG_OUT, info->nick, NF_ACCEPT, info->peer, p);
                }
                free(p);
            }

		}
        else if ((direction == PACKET_OUT) && (buf_cmp (line, "OUT", 3) == 0)) {
            info->logoned = 0;
            syslog (LOG_INFO, "%s sign out %08x,%d\n", info->nick, info->ip, info->port);
            send_im_event (MSN_IM, EVT_LOGOFF, info->nick, NF_ACCEPT, "", "");
            remove_client (info);
        }

        continue;
	}
    syslog (LOG_INFO, "MSG verdict return %d", ret);
	return ret; //(len - buf_len(&buf));
}

struct yhoo {
	u_char		version[8];
	u_int32_t	length;		/* all fields little-endian */
	u_int32_t	service;
	u_int32_t	connid;
	u_int32_t	magic;
	u_int32_t	unknown;
	u_int32_t	type;
	u_char		nick1[36];
	u_char		nick2[36];
};

struct ymsg {
	u_char		version[8];
	u_short		length;
	u_short		type;
	u_int32_t	unknown1;
	u_int32_t	unknown2;
};
enum YAHOO_SERVICE {
    YAHOO_SERVICE_LOGON             = 0x01,
    YAHOO_SERVICE_LOGOFF            = 0x02,
    YAHOO_SERVICE_MESSAGE           = 0x06,
    YAHOO_SERVICE_CONFMSG           = 0x1d,
    YAHOO_SERVICE_CHATLOGON         = 0x1e,
    YAHOO_SERVICE_CHATMSG           = 0x20,
    YAHOO_SERVICE_CHATJOIN          = 0x98,
    YAHOO_SERVICE_CHATLOGOUT        = 0xa0,
    YAHOO_SERVICE_COMMENT           = 0xa8
};

int
process_yahoo(struct client_info *info, u_char *data, int len, int direction, char *ip_local)
{
	struct ymsg *ymsg;
	struct buf *msg, buf;
	int i, reply;
    char nick1[MAX_NICK], nick2[MAX_NICK];
    char word[MAX_WORD];
    int ret = NF_ACCEPT;

    memset (word, 0, sizeof (word));
    memcpy (word, data, 20);

	syslog (LOG_INFO, "%d,%d-----YAHOOData=%s", direction, len, word);
	buf_init(&buf, data, len);
	
	if (buf_cmp(&buf, "YMSG", 4) == 0) {
		while (buf_len(&buf) > sizeof(*ymsg)) {
			ymsg = (struct ymsg *)buf_ptr(&buf);
			ymsg->length = ntohs(ymsg->length);
			ymsg->type = ntohs(ymsg->type);

			i = sizeof(*ymsg) + ymsg->length;

			if ((msg = buf_tok(&buf, NULL, i)) == NULL)
				break;
			
			buf_skip(msg, sizeof(*ymsg));
			
            reply = (direction == PACKET_IN);
            if (ymsg->type == YAHOO_SERVICE_LOGON) { 
                buf_skip(msg, 3); //skip "1"+0xc0+0xa8 direction
                buf_copyword (msg,  "\xc0\x80", 2, nick1, sizeof (nick1));
                if (info->nick) free (info->nick);
                info->nick = strdup (nick1);
                info->logoned = 1;
                syslog (LOG_INFO, "got yahoo logon %s", info->nick);
                send_im_event (YAHOO_IM, EVT_LOGON, info->nick, NF_ACCEPT, "", "");
                break; 
            }
            else if (ymsg->type == YAHOO_SERVICE_LOGOFF) {
                info->logoned = 0;
                syslog (LOG_INFO, "got yahoo logoff %s", info->nick);
                send_im_event (YAHOO_IM, EVT_LOGOFF, info->nick, NF_ACCEPT, "", "");
                remove_client (info);
                break; 
            }
            else if (ymsg->type == YAHOO_SERVICE_MESSAGE) {
    			buf_skip(msg, 3); //skip "1"+0xc0+0xa8  direction
    
                memset (nick1, 0, sizeof (nick1));
                buf_copyword (msg,  "\xc0\x80", 2, nick1, sizeof (nick1));
    			buf_skip(msg, 3);  //skip "5"+0xc0+0xa8
    
    			memset (nick2, 0, sizeof (nick2));
                buf_copyword (msg,  "\xc0\x80", 2, nick2, sizeof (nick2));
    
                //we'll only transmit msg <255
    			buf_skip(msg, 4); //skip "14"+0xc0+0xa8
    			memset (word, 0, sizeof (word));
                buf_copyword (msg,  "\xc0\x80", 2, word, sizeof (word));
    
    			if (strlen (nick1) && strlen (nick2) && strlen (word)) {
    				syslog (LOG_INFO, "%s Yahoo ", timestamp());
                    ret= checkBlockIMList (YAHOO_IM, nick2, ip_local);
    				if (reply) {
    					syslog (LOG_INFO, "%s > %s: %s\n",
    					       nick2, nick1, word);
                        send_im_event (YAHOO_IM, EVT_MSG_IN, nick1, ret, nick2, word);
                    }
    				else {
                        syslog (LOG_INFO, "%s > %s: %s\n",
                                nick1, nick2, word);
                        send_im_event (YAHOO_IM, EVT_MSG_OUT, nick1, ret, nick2, word);
                    }
    			}
                break;
            }
            else if (ymsg->type == YAHOO_SERVICE_CHATJOIN) {
                if (direction == PACKET_OUT) {
                    buf_skip(msg, 3); //skip "1"+0xc0+0xa8 direction

                    memset (nick1, 0, sizeof (nick1));
                    buf_copyword (msg,  "\xc0\x80", 2, nick1, sizeof (nick1));
                    buf_skip(msg, 5);   //skip "104"+0xc0+0xa8

                    memset (nick2, 0, sizeof (nick2));
                    buf_copyword (msg,  "\xc0\x80", 2, nick2, sizeof (nick2));

                    if (strlen (nick1) && strlen (nick2)) {
                        if (info->nick) free (info->nick);
                        info->nick = strdup (nick1);
                        info->logoned = 1;
                        syslog (LOG_INFO, "CHATJOIN %s want to join %s", nick1, nick2);
                        ret= checkBlockIMList (YAHOO_IM, nick2, ip_local);
                        send_im_event (YAHOO_CHAT, EVT_CHAT_JOIN, nick1, ret, nick2, "");
                    }
                }
                break;
            }
            else if (ymsg->type == YAHOO_SERVICE_COMMENT) {
                buf_skip(msg, 5);  //skip "104"+0xc0+0xa8

                memset (nick1, 0, sizeof (nick1));
                buf_copyword (msg,  "\xc0\x80", 2, nick1, sizeof (nick1));
                
                buf_skip(msg, 5); //skip "109"+0xc0+0xa8 who send
                memset (nick2, 0, sizeof (nick2));
                buf_copyword (msg,  "\xc0\x80", 2, nick2, sizeof (nick2));

                buf_skip(msg, 5); //skip "117"+0xc0+0xa8 content
    			memset (word, 0, sizeof (word));
                buf_copyword (msg,  "\xc0\x80", 2, word, sizeof (word));
                if (strlen (nick1) && strlen (nick2) && strlen (word)) {
                    ret= checkBlockIMList (YAHOO_IM, nick1, ip_local);
                    if (direction == PACKET_IN) {
                        syslog (LOG_INFO, "CHATMSG IN %s %s %s", nick1, nick2, word);
                        send_im_event (YAHOO_CHAT, EVT_CHAT_IN, nick1, ret, nick2, word);
                    }
                    else {
                        syslog (LOG_INFO, "CHATMSG OUT %s %s %s", nick1, nick2, word);
                        send_im_event (YAHOO_CHAT, EVT_CHAT_OUT, nick1, ret, nick2, word);
                    }
                }
                break;
            }
            else if (ymsg->type == YAHOO_SERVICE_CHATLOGOUT) {
                if (direction == PACKET_OUT) {
                    buf_skip(msg, 3);  //skip "1"+0xc0+0xa8
    
                    memset (nick1, 0, sizeof (nick1));
                    buf_copyword (msg,  "\xc0\x80", 2, nick1, sizeof (nick1));
                
                    if (strlen (nick1)) {
                        syslog (LOG_INFO, "CHAT LOGOUT %s", nick1);
                        send_im_event (YAHOO_CHAT, EVT_CHAT_EXIT, nick1, NF_ACCEPT, "", "");
                    }
                }
                break;
            }
            else if (ymsg->type == YAHOO_SERVICE_CONFMSG) {
                if (direction == PACKET_OUT) {
                    buf_skip(msg, 3); //skip "1"+0xc0+0xa8  direction
    
                    memset (nick1, 0, sizeof (nick1));
                    buf_copyword (msg,  "\xc0\x80", 2, nick1, sizeof (nick1));
                    
                    buf_skip(msg, 4);  //skip "57"+0xc0+0xa8   conf name
                    memset (word, 0, sizeof (word));
                    buf_copyword (msg,  "\xc0\x80", 2, word, sizeof (word));
                    //We only display the last nick name
                    while (buf_cmp(msg, "53", 2) == 0){
                        buf_skip(msg, 4);  //skip "53"+0xc0+0xa8  peername
                        memset (nick2, 0, sizeof (nick2));
                        buf_copyword (msg,  "\xc0\x80", 2, nick2, sizeof (nick2));
                        ret= checkBlockIMList (YAHOO_IM, nick2, ip_local);
                        if (ret != NF_ACCEPT) 
                            break;
                    }
                    if (ret != NF_ACCEPT) {
                        syslog (LOG_INFO, "YAHOO CONF BLOCKED");
                        send_im_event (YAHOO_IM, EVT_MSG_OUT, nick1, ret, nick2, "");
                        break;
                    }
                    //we'll only transmit msg <255
                    buf_skip(msg, 4); //skip "14"+0xc0+0xa8
                    memset (word, 0, sizeof (word));
                    buf_copyword (msg,  "\xc0\x80", 2, word, sizeof (word));
    
                    if (strlen (nick1) && strlen (nick2) && strlen (word)) {
                        syslog (LOG_INFO, "%s Yahoo ", timestamp());
                        syslog (LOG_INFO, "%s > %s: %s\n",
                                   nick2, nick1, word);
                        send_im_event (YAHOO_IM, EVT_MSG_OUT, nick1, ret, nick2, word);
                    }
                }
                else { //CONFMSG IN
                    buf_skip(msg, 3); //skip "1"+0xc0+0xa8  direction
    
                    memset (nick1, 0, sizeof (nick1));
                    buf_copyword (msg,  "\xc0\x80", 2, nick1, sizeof (nick1));
                    
                    buf_skip(msg, 4);  //skip "57"+0xc0+0xa8   conf name
                    memset (word, 0, sizeof (word));
                    buf_copyword (msg,  "\xc0\x80", 2, word, sizeof (word));

                    buf_skip(msg, 3);  //skip "3"+0xc0+0xa8  peername
                    memset (nick2, 0, sizeof (nick2));
                    buf_copyword (msg,  "\xc0\x80", 2, nick2, sizeof (nick2));

                    //we'll only transmit msg <255
                    buf_skip(msg, 4); //skip "14"+0xc0+0xa8
                    memset (word, 0, sizeof (word));
                    buf_copyword (msg,  "\xc0\x80", 2, word, sizeof (word));
    
                    if (strlen (nick1) && strlen (nick2) && strlen (word)) {
                        syslog (LOG_INFO, "%s Yahoo ", timestamp());
                        syslog (LOG_INFO, "%s > %s: %s\n",
                                   nick2, nick1, word);
                        send_im_event (YAHOO_IM, EVT_MSG_IN, nick1, ret, nick2, word);
                    }
                }
                break;
            }
            break;
		}
	}
    syslog (LOG_INFO, "YAHOO return %d", ret);
	return ret; 
}

#define MAX_LINE_PC         255
#define FILE_LANIP          "/var/etc/pc/lanip"
int get_lanip (char *ipbuf) 
{
    char buf[MAX_LINE_PC];
    FILE *fp;

    fp = fopen(FILE_LANIP, "r");
    if (fp != NULL) {
        if (fgets(buf, sizeof(buf) -1 , fp) != NULL) {
            char *s;
            s = strtok (buf," \t\n");
            strcpy (ipbuf, s);
            return 1;
        }
        fclose(fp);
    } 
    return 0;
}

/*
 * return Verdict of this packet
*/
int process_packet_msg (struct iphdr *tmp_iphdr, size_t packet_length)
{
    /* For parsing IP headers */
    struct udphdr *tmp_udphdr=
      (struct udphdr*) ((char*) tmp_iphdr+(tmp_iphdr->ihl*4));
    struct tcphdr *tmp_tcphdr=
      (struct tcphdr*) ((char*) tmp_iphdr+(tmp_iphdr->ihl*4));

    unsigned short srcport, dstport;
	struct client_info *c, *oldest_client;
    time_t oldest_time;
	int (*process_msgs)(struct client_info *, u_char *, int, int, char*);
	int is_new_client;
    static int first_time = 1;
    struct in_addr lan_addr;
    char lan_ip[20];
    in_addr_t local_mask = 0x0000ffff;
    in_addr_t local_addr = 0;
    int local_port = 0;
    int direction = 0;
    char *data;
    int  datalen;
    int ret = NF_ACCEPT;
    int curr_im_type = -1;
    char *existing_nick = NULL; 
    int num_clients = 0;
    struct in_addr saddr;

    /* We need the entire IP header, plus
          4 bytes for TCP/UDP source and destination ports. 
       or
          2 bytes for ICMP type and code. */
    unsigned int hdr_len = tmp_iphdr->ihl*4;

    if (first_time) {
        first_time = 0;
        init_client_list ();
        get_lanip (lan_ip);
        inet_aton (lan_ip, &lan_addr);
    }

    switch(tmp_iphdr->protocol)
      {
      case IPPROTO_UDP:
      case IPPROTO_TCP:
          hdr_len+=4;
        break;
      case IPPROTO_ICMP:
          hdr_len+=2;
      }

    if (hdr_len <= packet_length) {
        switch(tmp_iphdr->protocol) 
          {
          case IPPROTO_UDP:
            srcport = ntohs(tmp_udphdr->source);
            dstport = ntohs(tmp_udphdr->dest);
            break;
        case IPPROTO_TCP:
            data = (char *)tmp_tcphdr + tmp_tcphdr->doff * 4;
            datalen = htons (tmp_iphdr->tot_len) - (tmp_iphdr->ihl * 4 + tmp_tcphdr->doff * 4);
            srcport = ntohs(tmp_tcphdr->source);
            dstport = ntohs(tmp_tcphdr->dest);
            syslog (LOG_INFO, "------MSGSNIFF port=%d, %d, local=%08x,%08x,%08x,%08x, datalen=%d,%d", 
                    srcport, dstport, lan_addr.s_addr, lan_addr.s_addr & local_mask,
                    tmp_iphdr->saddr, tmp_iphdr->daddr, datalen, htons (tmp_iphdr->tot_len));
                
            if ((dstport >= 6660 && dstport <= 6680) ||
                (srcport >= 6660 && srcport <= 6680) ){
                process_msgs = process_irc;
                curr_im_type = MSN_CHAT;
            }
            else if ((srcport == 7012 || dstport == 7012) ){
                process_msgs = process_irc;
                curr_im_type = ICQ_CHAT;
            }
            else if (dstport == 5190 || srcport == 5190 || dstport == 9898 || srcport == 9898) {
                process_msgs = process_aim;
                curr_im_type = AOL_IM;
            }
            else if (dstport == 1863 || srcport == 1863) {
                process_msgs = process_msn;
                curr_im_type = MSN_IM;
            }
            else if (strncmp (data, "YMSG", 4) == 0) {
                process_msgs = process_yahoo;
                curr_im_type = YAHOO_IM;
            }
            else 
                return ret;

            if ((tmp_iphdr->saddr & local_mask) == (lan_addr.s_addr & local_mask)) 
            {
                local_addr = tmp_iphdr->saddr;
                local_port = srcport;
                direction = PACKET_OUT;
            }
            else
            {
                local_addr =  tmp_iphdr->daddr;
                local_port = dstport;
                direction = PACKET_IN;
            }
            //
            is_new_client = 1;
            oldest_client = NULL;
            oldest_time = time (0);
            num_clients = 0;
            c = client_list;
            existing_nick = NULL;
            while (c) {
                if (c->ip == local_addr && c->port == local_port) {
                    is_new_client = 0; 
                    c->active_time = time (0);
                    break;
                }
                //may use different port from the one of signon to send msg
                //since client is inserted to the list, the newer is at the front
                if (c->ip == local_addr && c->type == curr_im_type && c->nick &&!existing_nick) {
                    existing_nick = c->nick;
                }
                if (c->active_time < oldest_time) {
                    oldest_client = c;
                }
                num_clients ++;
                c = c->next;
            }

            if (is_new_client) {
                if ((c = malloc(sizeof(struct client_info))) == NULL) {
                    syslog (LOG_ERR, "No mem in sniff_msgs");
                    return 0;
                }
                memset ((char *)c, 0, sizeof(struct client_info));
                c->ip = local_addr;
                c->port = local_port;
                c->type = curr_im_type;
                c->active_time = time (0);
                saddr.s_addr = c->ip;
                strcpy (c->ip_str, inet_ntoa (saddr));
                if (existing_nick) { 
                    c->nick = strdup (existing_nick);
                    c->useExistingNick = 1;
                    syslog (LOG_INFO, "***************use existing nick %s", c->nick);
                }

                if (num_clients >= MAX_CLIENTS) {
                    syslog (LOG_INFO, "Too more clients %d, Remove client %08x, %d, %s", 
                            num_clients,
                            oldest_client->ip,
                            oldest_client->port,
                            oldest_client->nick);
                    remove_client (oldest_client);
                }
                insert_client (c);
                syslog (LOG_INFO, "num_client=%d,new client info %08x,%d", num_clients, local_addr, local_port);
            }
            syslog (LOG_INFO, "yahoo=%d,newclient= %d, %08x, offset=%d ,tot_len=%d, datalen=%d, %d", 
                    process_msgs == process_aim,
                    is_new_client, local_addr, tmp_tcphdr->doff * 4, htons (tmp_iphdr->tot_len), 
                    htons (tmp_iphdr->tot_len) - (tmp_iphdr->ihl * 4 + tmp_tcphdr->doff * 4), 
                    datalen);
            ret = process_msgs(c, data, datalen, direction, c->ip_str);

            break;
          default:
            srcport = dstport = 0;
            break;
          }
    }
    return ret;
}

