#!/usr/local/bin/perl

print "/* apps/progs.h */\n";
print "/* automatically generated by progs.pl for openssl.c */\n\n";
print "/* 2002-10-10 MWi <martin.witzel\@de.ibm.com>:        */\n";
print "/* Disabled most functions for small devices, except */\n";
print "/* genrsa, req, s_client, s_server, version.         */\n";
print "/* This assumes that only RSA is enabled on small    */\n";
print "/* devices but not DSA or DH. The build process for  */\n";
print "/* small devices enforces this.                      */\n\n";

grep(s/^asn1pars$/asn1parse/,@ARGV);

foreach (@ARGV)
	{ printf "extern int %s_main(int argc,char *argv[]);\n",$_; }

print <<'EOF';

#define FUNC_TYPE_GENERAL	1
#define FUNC_TYPE_MD		2
#define FUNC_TYPE_CIPHER	3

typedef struct {
	int type;
	char *name;
	int (*func)();
	} FUNCTION;

FUNCTION functions[] = {
EOF

# Selectively enable certain key and certificate handling functions on small devices
# Here we enable only: genrsa, req, s_client, s_server, version. All other functions
# are considered superfluous.
# mwitzel@de.ibm.com
print "/* --- general functions in openssl --- */\n";
foreach (@ARGV)
	{
	push(@files,$_);
	$str="\t{FUNC_TYPE_GENERAL,\"$_\",${_}_main},\n";
	# --- Allow the s_client, s_server function on small devices (but not s_time_main and other s_*)
	if ( ($_ =~ /^s_client/) || ($_ =~ /^s_server/) ) 
		{ print "#if !defined(OPENSSL_NO_SOCK) && !(defined(OPENSSL_NO_SSL2) && defined(OPENSSL_NO_SSL3))\n${str}#endif\n"; } 
	elsif ($_ =~ /^s_time_main/)
		{ print "#if !defined(OPENSSL_NO_SOCK) && !(defined(OPENSSL_NO_SSL2) && defined(OPENSSL_NO_SSL3))\n #ifndef OPENSSL_EMBED\n${str} #endif\n#endif\n"; } 
	elsif ($_ =~ /^ciphers$/)
		{ print "#if !defined(OPENSSL_NO_SOCK) && !(defined(OPENSSL_NO_SSL2) && defined(OPENSSL_NO_SSL3))\n #ifndef OPENSSL_EMBED\n${str} #endif\n#endif\n"; } 
	elsif ( ($_ =~ /^rsa$/) || ($_ =~ /^rsautl$/)) 
		{ print "#ifndef OPENSSL_NO_RSA\n #ifndef OPENSSL_EMBED\n${str} #endif\n#endif\n";  }
	# --- Allow the genrsa function on small devices
	elsif ( $_ =~ /^genrsa$/ )
		{ print "#ifndef OPENSSL_NO_RSA\n${str}#endif\n";  }
	elsif ( ($_ =~ /^dsa$/) || ($_ =~ /^gendsa$/) || ($_ =~ /^dsaparam$/))
		{ print "#ifndef OPENSSL_NO_DSA\n #ifndef OPENSSL_EMBED\n${str} #endif\n#endif\n"; }
	elsif ( ($_ =~ /^dh$/) || ($_ =~ /^gendh$/) || ($_ =~ /^dhparam$/))
		{ print "#ifndef NO_DH\n #ifndef OPENSSL_EMBED\n${str} #endif\n#endif\n"; }
	elsif ( ($_ =~ /^pkcs12$/))
		{ print "#if !defined(OPENSSL_NO_DES) && !defined(OPENSSL_NO_SHA1)\n #ifndef OPENSSL_EMBED\n${str} #endif\n#endif\n"; }
	# --- Allow the req, version functions on small devices 
	elsif ( ($_ =~ /^req$/) || ($_ =~ /^version$/) ) 
		{ print $str; }
	else
		{ print "#ifndef OPENSSL_EMBED\n${str}#endif /* OPENSSL_EMBED */\n"; }
	}

# Disable all digest functions in the openssl executable on small devices
print "\n/* --- digest functions in openssl --- */\n";
print "#ifndef OPENSSL_EMBED\n";
foreach ("md2","md4","md5","sha","sha1","mdc2","rmd160")
	{
	push(@files,$_);
	printf " #ifndef OPENSSL_NO_".uc($_)."\n\t{FUNC_TYPE_MD,\"".$_."\",dgst_main},\n #endif\n";
	}
print "#endif /* OPENSSL_EMBED */\n";

print "\n/* --- cipher functions in openssl --- */\n";
foreach (
	"aes-128-cbc", "aes-128-ecb",
	"aes-192-cbc", "aes-192-ecb",
	"aes-256-cbc", "aes-256-ecb",
	"base64",
	"des", "des3", "desx", "idea", "rc4", "rc4-40",
	"rc2", "bf", "cast", "rc5",
	"des-ecb", "des-ede",    "des-ede3",
	"des-cbc", "des-ede-cbc","des-ede3-cbc",
	"des-cfb", "des-ede-cfb","des-ede3-cfb",
	"des-ofb", "des-ede-ofb","des-ede3-ofb",
	"idea-cbc","idea-ecb",   "idea-cfb", "idea-ofb",
	"rc2-cbc", "rc2-ecb", "rc2-cfb","rc2-ofb", "rc2-64-cbc", "rc2-40-cbc",
	"bf-cbc",  "bf-ecb",     "bf-cfb",   "bf-ofb",
	"cast5-cbc","cast5-ecb", "cast5-cfb","cast5-ofb",
	"cast-cbc", "rc5-cbc",   "rc5-ecb",  "rc5-cfb",  "rc5-ofb")
	{
	push(@files,$_);

	# Disable all cipher functions in the openssl executable on small devices
	$t=sprintf("\t{FUNC_TYPE_CIPHER,\"%s\",enc_main},\n",$_);
	if    ($_ =~ /des/)  { $t="#ifndef OPENSSL_EMBED\n #ifndef OPENSSL_NO_DES\n${t} #endif\n#endif\n"; }
	elsif ($_ =~ /aes/)  { $t="#ifndef OPENSSL_EMBED\n #ifndef OPENSSL_NO_AES\n${t} #endif\n#endif\n"; }
	elsif ($_ =~ /idea/) { $t="#ifndef OPENSSL_EMBED\n #ifndef OPENSSL_NO_IDEA\n${t} #endif\n#endif\n"; }
	elsif ($_ =~ /rc4/)  { $t="#ifndef OPENSSL_EMBED\n #ifndef OPENSSL_NO_RC4\n${t} #endif\n#endif\n"; }
	elsif ($_ =~ /rc2/)  { $t="#ifndef OPENSSL_EMBED\n #ifndef OPENSSL_NO_RC2\n${t} #endif\n#endif\n"; }
	elsif ($_ =~ /bf/)   { $t="#ifndef OPENSSL_EMBED\n #ifndef OPENSSL_NO_BF\n${t} #endif\n#endif\n"; }
	elsif ($_ =~ /cast/) { $t="#ifndef OPENSSL_EMBED\n #ifndef OPENSSL_NO_CAST\n${t} #endif\n#endif\n"; }
	elsif ($_ =~ /rc5/)  { $t="#ifndef OPENSSL_EMBED\n #ifndef OPENSSL_NO_RC5\n${t} #endif\n#endif\n"; }
	elsif ($_ =~ /base64/)  { $t="#ifndef OPENSSL_EMBED\n${t}#endif\n"; }
	print $t;
	}

print "\t{0,NULL,NULL}\n\t};\n";
