#!/usr/bin/wish 

# execlog - run a program with exec and log the output
# Set window title
wm title . "TI Linux Gateway"

# Create a frame for buttons and entry.
frame .top -borderwidth 10
pack .top -side top -fill x

# Create the command buttons.

button .top.quit -text Quit -command exit
set but [button .top.run -text "Build" -command Run]
pack .top.quit .top.run -side right
button .top.go -text "Guage" -command {
    for {set i 0} {$i <= 100} {incr i} {
        after 100
        gauge_value .g $i
    }
}
pack .top.go -side bottom -side right

# Create a text widget to log the output

frame .t -relief groove
set log [text .t.log -width 80 -height 10 -background black -foreground green \
	-borderwidth 2 -relief raised -setgrid true \
	-yscrollcommand {.t.scroll set}]
scrollbar .t.scroll -command {.t.log yview}
pack .t.scroll -side right -fill y
pack .t.log -side left -fill both -expand true
pack .t -side top -fill both -expand true

frame .t.options -borderwidth 10
pack .t.options -side left -fill x

# Create a text widget to provide build status
frame .build_stat
set log_bs [text .build_stat.log_bs -width 80 -height 1 -background gray -foreground blue \
	-borderwidth 2 -relief raised -setgrid true]
pack .build_stat.log_bs -side left -fill both -expand true
pack .build_stat -side top -fill both -expand true


# Board options

frame .t.options.board -borderwidth 3 -relief groove
label .t.options.board.label -text boards -background black -foreground green
pack .t.options.board  -fill x
radiobutton .t.options.board.ar5d01 -variable board -text "AR5D01 " -value ar5d01
radiobutton .t.options.board.acpep -variable board -text "ACPEP  " -value acpep
radiobutton .t.options.board.tnetwa100 -variable board -text "WA100  " -value wa100
grid .t.options.board.label -row 0 -column 0
grid .t.options.board.ar5d01 -row 1 -column 0
grid .t.options.board.acpep  -row 2 -column 0
grid .t.options.board.tnetwa100 -row 3 -column 0



# Distribution options

frame .t.options.dist -borderwidth 3 -relief groove
label .t.options.dist.label -text distribution -background black -foreground green
pack .t.options.dist  -fill x -expand true
radiobutton .t.options.dist.nsp -variable dist -text "NSP     " -value nsp
radiobutton .t.options.dist.psp -variable dist -text "PSP     " -value psp
radiobutton .t.options.dist.custom -variable dist -text Custom -value custom -command custom_win
grid .t.options.dist.label -row 0 -column 0
grid .t.options.dist.nsp -row 1 -column 0
grid .t.options.dist.psp -row 2 -column 0
grid .t.options.dist.custom -row 3 -column 0


# filesystem options

frame .t.options.filesystem -borderwidth 3 -relief groove
label .t.options.filesystem.label -text filesystem -background black -foreground green
pack .t.options.filesystem  -fill x
radiobutton .t.options.filesystem.cramfs -variable filesystem -text CRAMFS -value cramfs
radiobutton .t.options.filesystem.jffs2 -variable filesystem -text "JFFS2   " -value jffs2
grid .t.options.filesystem.label -row 0 -column 0
grid .t.options.filesystem.cramfs -row 1 -column 0
grid .t.options.filesystem.jffs2  -row 2 -column 0


# Build the command string to be sent to the shell prompt

proc Build_Command_String {} {
    global make_command_string filesystem dist board
    switch -exact -- $board {
        "ar5d01" { set make_command_string "make $dist BOARD=$board" }
        "acpep"  { set make_command_string "make $dist BOARD=$board FS=$filesystem" }
        "wa100"  { set make_command_string "make $dist BOARD=$board FS=$filesystem" }
    }
}


#*-------------------------------------------------------------------*
#*  TI Linux Distribution Version 3.0.0
#*  Sun Dec  1 16:17:32 EST 2002
#*  Making RPM: TI_Linux-target-uclibc-0.9.12-TI3.0.src.rpm
#*
#*--------------------------------------------------------------------*

proc Parse_Header_Strings {line_string} {
    global rpm_name
    if [regexp {([^:]+)Making RPM: ([^: ]+)} $line_string match match2 rpm_name] { 
        regsub -all {\)\)} $rpm_name "" rpm_name
        return $rpm_name
    }
    return 0
}



# Run the program and arrange to read its input

proc Run {} {
	global command input log but make_command_string
        Build_Command_String
    if [catch {open "|$make_command_string |& cat"} input] {
		$log insert end $input\n
	} else {
		fileevent $input readable Log
		$log insert end $make_command_string\n
		$but config -text Stop -command Stop
	}
}


set index_log_bs 0
# Process text from Command line for information
proc Process_Log {line_string} {
    global source_rpm_name log_bs index_log_bs 
#    puts stdout "$line_string"
    catch {Parse_Header_Strings ($line_string)} source_rpm_name
    if [string equal $source_rpm_name "0"] {
    } else {
        incr index_log_bs +1
        puts stdout "Making $source_rpm_name"
        $log_bs insert end "Making $source_rpm_name\n"
        puts stdout "$index_log_bs"
        $log_bs see $index_log_bs.0
    }
}


# Read and log output from the program

proc Log {} {
	global input log
	if [eof $input] {
		Stop
	} else {
		gets $input line
                Process_Log ($line)
		$log insert end $line\n
		$log see end
	}
}

# Stop the program and fix up the button

proc Stop {} {
	global input but
	catch {close $input}
	$but config -text "Build" -command Run
}

# Progress Guage

option add *Gauge.borderWidth 2 widgetDefault
option add *Gauge.relief sunken widgetDefault
option add *Gauge.length 700 widgetDefault
option add *Gauge.color gray widgetDefault

proc gauge_create {win {color ""}} {
    frame $win -class Gauge

    set len [option get $win length Length]

    canvas $win.display -borderwidth 0 -background gray \
        -highlightthickness 0 -width $len -height 20
    pack $win.display 

    if {$color == ""} {
        set color [option get $win color Color]
    }
    $win.display create rectangle 0 0 0 20 \
        -outline "" -fill $color -tags bar
    $win.display create text [expr 0.5*$len] 10 \
        -anchor c -text "0%" -tags value

    return $win
}

proc gauge_value {win val} {
    if {$val < 0 || $val > 100} {
        error "bad value \"$val\": should be 0-100"
    }
    set msg [format "%3.0f%%" $val]
    $win.display itemconfigure value -text $msg

    set w [expr 0.01*$val*[winfo width $win.display]]
    set h [winfo height $win.display]
    $win.display coords bar 0 0 $w $h

    update
}

# Progress Guage
gauge_create .g Blue
pack .g  -fill both -padx 10 -pady 10 -side bottom

#----------------------------------
# Distribution customization window
#----------------------------------
toplevel .top_custom
wm title .top_custom "Custom Distribution"

frame .top_custom.applications -borderwidth 3 -relief groove
label .top_custom.applications.label -text distribution -background black -foreground green
checkbutton .top_custom.applications.busybox -text BusyBox -command {custom_busybox}
checkbutton .top_custom.applications.telnet -text Telnet
checkbutton .top_custom.applications.br2684 -text br2684
checkbutton .top_custom.applications.cfgmgr -text CfgMgr
checkbutton .top_custom.applications.dproxy -text DNS-proxy
checkbutton .top_custom.applications.iptables -text IPtables
checkbutton .top_custom.applications.arp -text ARP
checkbutton .top_custom.applications.pppd -text PPPd
checkbutton .top_custom.applications.pppoe -text PPPoE
checkbutton .top_custom.applications.thttpd -text "Tiny Webserver"
checkbutton .top_custom.applications.udhcp -text "DHCP client/server"
checkbutton .top_custom.applications.dhcpfwd -text "DHCP Forwarder"
grid .top_custom.applications.busybox   -row 0 -column 0 -sticky w
grid .top_custom.applications.telnet    -row 0 -column 1 -sticky w
grid .top_custom.applications.br2684    -row 1 -column 0 -sticky w
grid .top_custom.applications.cfgmgr    -row 1 -column 1 -sticky w
grid .top_custom.applications.dproxy    -row 2 -column 0 -sticky w
grid .top_custom.applications.iptables  -row 2 -column 1 -sticky w
grid .top_custom.applications.arp       -row 3 -column 0 -sticky w
grid .top_custom.applications.pppd      -row 3 -column 1 -sticky w
grid .top_custom.applications.pppoe     -row 4 -column 0 -sticky w
grid .top_custom.applications.thttpd    -row 4 -column 1 -sticky w
grid .top_custom.applications.udhcp     -row 5 -column 0 -sticky w
grid .top_custom.applications.dhcpfwd   -row 5 -column 1 -sticky w
pack .top_custom.applications -fill x -padx 4 -pady 4
button .top_custom.ok -text "Save Configuration" -command {wm withdraw .top_custom}
pack .top_custom.ok -side bottom -pady 8
focus .top_custom.ok
wm withdraw .top_custom


#-------------------------------------
# Busybox customization window
#------------------------------------
toplevel .busybox
wm title .busybox "Busybox custom configuration"
frame .busybox.selection  -borderwidth 3 -relief groove
label .busybox.selection.icon -bitmap questhead
label .busybox.selection.mesg -text "Customize busybox during the build process?"
pack .busybox.selection.icon -side left
pack .busybox.selection.mesg -side right -expand yes
frame .busybox.sep -height 2 -borderwidth 1 -relief sunken
frame .busybox.controls
button .busybox.controls.yes -text "Yes" -command {yes_custom_busybox} 
button .busybox.controls.no -text "No" -command {wm withdraw .busybox}
pack .busybox.controls.yes -side left -padx 4 -expand yes
pack .busybox.controls.no -side left -padx 4 -expand yes
pack .busybox.selection -padx 8 -pady 8 -expand yes -fill both
pack .busybox.sep -fill x -pady 4
pack .busybox.controls -pady 4 -fill x
focus .busybox.controls.no
wm withdraw .busybox



proc custom_win {} {
wm deiconify .top_custom
}

proc custom_busybox {} {
wm deiconify .busybox
}

proc yes_custom_busybox {} {
wm withdraw .busybox

}
